#include <QPainter>
#include <QTimer>
#include <QCoreApplication>
#include <QString>
#include <QStringList>
#include <QKeyEvent>

#include "inc/Window.h"
#include "inc/csvparser.h"
#include "inc/gravity_object.h"

Window::Window(QWidget* parent) : QWidget(parent) {
    simulation_step_counter = 0;

    // get csv file from command line option
    QStringList arguments = QCoreApplication::arguments();
    QString input_filename = arguments.at(1);
    CsvParser csv(input_filename, this);

    // initialize objects vector with values from csv file
    QStringList ElementProperties = csv.next();
    while (ElementProperties.length() > 0) {
        QVector<qreal> pos = {ElementProperties.at(0).toDouble(), ElementProperties.at(1).toDouble()};
        QVector<qreal> vel = {ElementProperties.at(2).toDouble(), ElementProperties.at(3).toDouble()};
        qreal mass = ElementProperties.at(4).toDouble();

        objects.append(new gravity_object(this));
        objects.last()->initialize(pos, vel, mass);
        ElementProperties = csv.next();
    }

    // initialize and start sysTick timer
    sysTickTimer = new QTimer(this);
    connect(sysTickTimer, SIGNAL(         timeout()),
                    this, SLOT(   sysTickCallback()));
    sysTickTimer->start(10);
}

void Window::paintEvent(QPaintEvent*) {
    QPainter painter(this);
    QRect viewport = painter.viewport();
    painter.fillRect(viewport, QColor(255, 255, 255));
    painter.drawText(viewport.topLeft() + QPoint(5, 15),
                     "Step: " + QString::number(simulation_step_counter));

    for (gravity_object* object : objects) {
        object->paint(&painter);
    }
}

void Window::keyPressEvent(QKeyEvent* event){
    if (event->key() == Qt::Key_Space) {
        if (sysTickTimer->isActive()) {
            sysTickTimer->stop();
        } else {
            sysTickTimer->start();
        }
    }
}

void Window::sysTickCallback(void) {
    // calculate all new velocities
    for (gravity_object* object_a : objects) {
        for (gravity_object* object_b : objects) {
            object_a->update_velocity_influenced_by(object_b);
        }
    }

    // calculate all new positions
    for (gravity_object* object : objects) {
        object->update_position();
    }

    // update simulation step counter
    simulation_step_counter++;

    // repaint output
    update();
}
